/** DataNode.java
 *  Copyright (c) 2001 Donna Malayeri
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package edu.umd.cs.fuzzyTree;

import java.lang.*;
import java.util.*;

/** Encapsulates the information that is in a node in an XML document.
 *  A node has a label and data.  A preorder index is also assigned, which is the value
 *  assigned to the node in a preorder traversal of the tree.
 *
 *  @author     Donna Malayeri
 *  @version    24-Jul-01
 */

class DataNode
{
	private String label;
	private String data;

	/** Index of this node in the DataGraph */
	private int graphIndex;

	/** Index of this node's parent in the DataGraph */
	int parentGraphIndex = -1;

	/** Index of the node in a preorder traversal of the tree */
	int preorderIndex = -1;

	/** Reference to the display node corresponding to this <code>DataNode</code>. Value may be
	 *  null, in which case the node is not displayed in the display tree.  If non-null,
	 *  <code>displayNode.dataNode == this</code>.
	 */
	public DisplayNode displayNode;

	/** Index assigned for comparing whether two nodes are isomorphic to one another */
	int isomorphismIndex = -1;

	/** Size of this node (1) + size of node's subtree (a leaf has size 1). */
	int treeSize;

	/** This node's rank in its equivalance class.
	 *  Value is between 1 (highest rank) and the size of the equivalence class.
	 */
	int rankInClass;

	/** The node's score, which is a measure of its desirability of being in the display tree */
	int score;

	/** Constructs a <code>DataNode</code> with the given label, data, and index into the
	 *  data graph.
	 */
	public DataNode(String label, String data, int graphIndex)
	{
		this.label = label;

		if (data != null)
		{
			this.data = data.trim();
			data.replace('\n', ' ');
		}

		this.graphIndex = graphIndex;
	}

	/** Returns this node's index into the DataGraph */
	public int getGraphIndex()
	{
		return graphIndex;
	}

	/** Returns the node's label */
	public String getLabel()
	{
		return label;
	}

	/** Returns the node's data  */
	public String getData()
	{
		return data;
	}

	/** Returns a string representation of the data node, which includes preorder index,
	 *  label, data, and score.
	 */
	public String toString()
	{
		// "<html><font face=verdana size=-1>"
		return preorderIndex + "\t" + getLabel() + "; " + getData() + "     " + score;
	}

	/** Set the index of this node's parent into the DataGraph. */
	public void setParentGraphIndex(int parent)
	{
		parentGraphIndex = parent;
	}

	/** Returns this node's parent, using its <code>parentGraphIndex</code> and the <code>dataGraph</code>.
	 *  Null is returned if this node is the root.
	 */
	public DataNode getParent(DataGraph dataGraph)
	{
		return parentGraphIndex >= 0 ? dataGraph.getNode(parentGraphIndex) : null;
	}
}

