/** MovingSlider.java
 *  Copyright (c) 2001 Donna Malayeri
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package edu.umd.cs.fuzzyTree;

import javax.swing.*;
import java.awt.event.*;
import javax.swing.event.*;
import java.awt.*;

/** Slider that moves the labels and slider position as the user
 *  moves the slider to the right or to the left, to allow for a greater range of values than
 *  that of a regular slider.  For example, if the user moves the slider from 20 to 40, when there
 *  is an actual maximum of 100, and a displayed maximum of 50, the slider labels might move
 *  to the right so 70 would be visible.
 *
 *  <p>The displayed minimum will always be a multiple of the major tick spacing. The display
 *  range must be at least equal to the difference between the real maximum and the real minimum.
 *
 *
 *  @author     Donna Malayeri
 *  @version    24-Jul-01
 */

public class MovingSlider extends JSlider
{
	/** the range of values to be displayed */
	private int displayRange;

	/** the minimum number of values to the left of the slider position that should be displayed */
	private int leftMargin;

	/** the minimum number of values to the right of the slider position that should be displayed */
	private int rightMargin;

	/** the actual, or virtual, minimum of the slider */
	private int realMinimum;
	/** the actual, or virtual, maximum of the slider */
	private int realMaximum;

	/** the previous slider value, used to determine the direction the slider was moved */
	private int oldValue = 0;

	/** whether or not the value of 0 is allowed */
	public boolean zeroNotAllowed = true;

	public MovingSlider(int minValue, int maxValue, int value, int displayRange,
						int leftMargin, int rightMargin)
	{
		this.displayRange = displayRange;
		this.leftMargin = leftMargin;
		this.rightMargin = rightMargin;

		setMinimum(value);
		setMaximum(value + displayRange);
		realMinimum = minValue;
		realMaximum = maxValue;

		adjustValue(value);
		init();
	}

	/** Constructs a moving slider with an actual minimum of 0, an actual maximum of 100,
	 *  initial value of 0, and left and right margin of 10.
	 */
	public MovingSlider()
	{
		this(0, 100, 0, 50, 10, 10);
		init();
	}

	/** Sets the current value of the slider to be <code>newValue</code>. The displayed
	 *  maximum or minimum may change as a result of the operation.
	 *  @param newValue     new slider value
	 */
	public void adjustValue(int newValue)
	{
		setToolTipText(newValue + "");

		int newMin, newMax;

		if (oldValue < newValue)    // moving slider forward
		{
			newMin = Math.max(realMinimum, newValue - leftMargin);
			newMax = newMin + displayRange;

			if (newMax > realMaximum)
			{
				newMin -= newMax - realMaximum;
				newMax = realMaximum;
			}
		}

		else    // moving slider backward
		{
			newMax = Math.min(realMaximum, newValue + rightMargin);
			newMin = newMax - displayRange;

			if (newMin < realMinimum)
			{
				newMax += realMinimum - newMin;
				newMin = realMinimum;
			}
		}

		if ( (newMin % 5) != 0)
		{
			int valIncrement = 5 - (newMin % 5);
			newMin += valIncrement;
			newMax += valIncrement;
		}

		if (newValue == getValue())
			oldValue = newValue;

		setValueIsAdjusting(true);
		setMinimum(newMin);
		setMaximum(newMax);

		if (newValue != getValue())
		{
			oldValue = newValue;
			setValue(newValue);
		}

		setValueIsAdjusting(false);
	}

	private void init()
	{
		setPaintTicks(true);
		setPaintLabels(true);
		setSnapToTicks(true);
		oldValue = getValue();

		addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e)
			{
				JSlider source = (JSlider)e.getSource();
				if (!source.getValueIsAdjusting() && source.getValue() != oldValue)
				{
//					System.out.println("stateChanged called, getValue: " + source.getValue() +
//						" oldValue: " + oldValue);

					int value = source.getValue();

					if (value > 0)
						adjustValue(value);

					else if (zeroNotAllowed)
						source.setValue(1);
				}

			}
		});
	}

	/** Returns the minimum number of values to the left of the slider position that should
	 *  be displayed.
	 *  @return the left margin
	 */
	public int getLeftMargin()
	{
		return leftMargin;
	}

	/** Sets the minimum number of values to the left of the slider position that should
	 *  be displayed. This value will be used, unless the slider is less than <code>leftMargin</code>
	 *  away from the actual minimum, or if a displayed minimum must be chosen so as to be a
	 *  multiple of <code>majorTickSpacing</code>.
	 *  @param newLeftMargin    new value for left margin
	 */
	public void setLeftMargin(int newLeftMargin)
	{
		leftMargin = newLeftMargin;
	}

	/** Returns the minimum number of values to the right of the slider position that
	 *  should be displayed.
	 *  @return the right margin
	 *  */
	public int getRightMargin()
	{
		return rightMargin;
	}

	/**  Sets the minimum number of values to the right of the slider position that
	 *  should be displayed.
	 *  This value will be used, unless the slider is less than <code>rightMargin</code>
	 *  away from the actual maximum, or if a displayed minimum must be chosen so as to be a
	 *  multiple of <code>majorTickSpacing</code>.
	 *  @param newRightMargin   the new right margin
	 */
	public void setRightMargin(int newRightMargin)
	{
		rightMargin = newRightMargin;
	}

	/** Returns the range of values to be displayed, or the difference between
	 *  <code>getMaximum()</code> and <code>getMinimum</code>.
	 *  @return the display range
	 *  */
	public int getDisplayRange()
	{
		return displayRange;
	}

	/** Sets the range of values to be displayed.  This must be at least the difference between
	 *  the actual maximum and the actual minimum.
	 */
	public void setDisplayRange(int newDisplayRange)
	{
		displayRange = newDisplayRange;
	}

	/** Returns the actual minimum value for the slider. */
	public int getRealMinimum()
	{
		return realMinimum;
	}

	/** Sets the actual minimum value for the slider. */
	public void setRealMinimum(int newRealMinimum)
	{
		realMinimum = newRealMinimum;
	}

	/** Returns the actual maximum value for the slider. */
	public int getRealMaximum()
	{
		return realMaximum;
	}

	/** Sets the actual maximum value for the slider */
	public void setRealMaximum(int newRealMaximum)
	{
		realMaximum = newRealMaximum;
		if (newRealMaximum > getMaximum())
			setMaximum(newRealMaximum);
	}

	/** Returns whether or not the slider allows a value of 0. */
	public boolean getZeroNotAllowed()
	{
		return zeroNotAllowed;
	}

	/** Sets whether or not the slider allows a value of 0. */
	public void setZeroNotAllowed(boolean newZeroNotAllowed)
	{
		zeroNotAllowed = newZeroNotAllowed;
	}

	public static void main(String[] args)
	{
		JFrame frame = new JFrame();
		final MovingSlider movingSlider1 = new MovingSlider(0, 75, 60, 50, 10, 10);
		movingSlider1.setMajorTickSpacing(5);
		movingSlider1.setMinorTickSpacing(1);
		movingSlider1.setPreferredSize(new Dimension(500, movingSlider1.getPreferredSize().height));

		frame.addWindowListener(new WindowAdapter()
		{
			public void windowClosing(WindowEvent e) {
				System.exit(0);
			}
		});

		movingSlider1.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e)
			{
				JSlider source = (JSlider)e.getSource();
				if (!source.getValueIsAdjusting())
				{
					int value = (int)source.getValue();

					if (value > 0)
						System.out.println("Value: " + value);
				}
			}
		});

		frame.getContentPane().setLayout(new BorderLayout());
		frame.getContentPane().add(movingSlider1, BorderLayout.NORTH);

		JButton button = new JButton("Change");
		button.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				movingSlider1.adjustValue(20);
			}
		});

		frame.getContentPane().add(button, BorderLayout.SOUTH);
		frame.pack();
		frame.show();
	}
}
