/** ScoreGenerator.java
 *  Copyright (c) 2001 Donna Malayeri
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package edu.umd.cs.fuzzyTree;

import java.util.*;
import java.io.PrintWriter;

/** Creates scores from an tree of <code>DataNode</code>s.  Nodes are assigned
 *  scores based on how desirable they are to be displayed.  A node's score
 *  is the product of the size of the subtree of which it is the root and the number
 *  of subtrees isomorphic to it, plus some additional parameters. A list that is
 *  sorted based on these scores is the final product.
 *
 *  @author     Donna Malayeri
 *  @version    24-Jul-01
 */

public class ScoreGenerator
{
	/** Reference to the XML data graph */
	private DataGraph _dataGraph;

	/** An <code>ArrayList</code> of <code>ArrayList</code> of <code>DataNode</code>s.
	 *  <code>_tempNodeList[i]</code> contains nodes which have height <code>i</code>.
	 */
	private ArrayList _tempNodeList = new ArrayList();

	/** Used in preprocessing; the highest score value. */
	private int _maxInitialScore = 0;

	/** Used in preprocessing; the lowest score value. Used to ensure all scores are positive. */
	private int _minInitialScore = 0;

	/** The minimum score value that a node may have. */
	public static final int MIN_NODE_SCORE = 10;

	/** A sorted list of all of the <code>DataNode</code>s in the <code>DataGraph</code>.
	 *  Nodes are sorted based on the score field.
	 */
	private ArrayList _sortedNodeList = new ArrayList();   // ArrayList<DataNode>

	/** Comparator for sorting on descending order of score */
	public Comparator descendingScoreComparator =
		new Comparator() {
				public int compare(Object o1, Object o2)
				{
					return ((DataNode)o2).score - ((DataNode)o1).score;
				}
		};

	/** Creates a score processor using <code>dataGraph</code>, performs all processing and
	 *  creates a sorted node list, which can be retrieved using {@link #getSortedNodeList}.
	 */
	public ScoreGenerator(DataGraph dataGraph)
	{
		_dataGraph = dataGraph;

		// load height member of each DataNode, excluding the root
		Iterator rootChildrenIter = dataGraph.getEdges(dataGraph.getRoot()).iterator();
		while (rootChildrenIter.hasNext())
			calculateHeightAndCreateLists((DataNode)rootChildrenIter.next());

		// create equivalence classes, assign initial scores to nodes, copy to _sortedNodeList
		createEquivalenceClasses();

		// set the rank parameter for node scores
		setRankNodeScores();

		// sort the node list in descending order
		Collections.sort(_sortedNodeList, descendingScoreComparator);
	}

	/** Returns an <code>ArrayList</code> of <code>DataNode</code>s that are sorted in
	 *  descending order of score
	 */
	public ArrayList getSortedNodeList()
	{
		return _sortedNodeList;
	}

	/** Creates equivalance classes for nodes based on an isomormphism index ordering.
	 *  IsomorphicNodeComparator is used for the ordering.
	 *  @see edu.umd.cs.fuzzyTree.IsomorphicDataNodeComparator
	 */
	private void createEquivalenceClasses()
	{
		Comparator isoComparator = new IsomorphicDataNodeComparator(_dataGraph);
		int currentIsoIndex = -1;

		for (int i = 0; i < _tempNodeList.size(); i++)
		{
			ArrayList list = (ArrayList) _tempNodeList.get(i);
			Collections.sort(list, isoComparator);

			// set isomorphism indices now
			DataNode lastNode = null;
			int lastEquivClassStart = -1;

			for (int j = 0; j < list.size(); j++)
			{
				DataNode currentNode = (DataNode) list.get(j);

				// if we're on the first node of the list, or if the current node
				// isn't isomorphic to the last node, then get a new index
				if (j == 0 || isoComparator.compare(lastNode, currentNode) != 0)
				{
					currentNode.isomorphismIndex = ++currentIsoIndex;
					currentNode.rankInClass = 1;

					if (lastEquivClassStart >= 0)
					{
						// compute scores of last equivalence class, because its size is now known
						setInitialScores(lastEquivClassStart, _sortedNodeList.size());
					}

					lastEquivClassStart = _sortedNodeList.size();
				}
				else
				{
					currentNode.isomorphismIndex = currentIsoIndex;
					currentNode.rankInClass = lastNode.rankInClass + 1;
				}

				lastNode = currentNode;
				_sortedNodeList.add(currentNode);
			}

			setInitialScores(lastEquivClassStart, _sortedNodeList.size());

			list.clear();
		}

		_tempNodeList.clear();
		_tempNodeList = null;
	}

	/** Sets the initial scores of nodes in <code>_sortedNodeIndex</code> from
	 *  <code>startIndex</code> to <code>onePastEndIndex</code>, which mark the start and end
	 *  of one equivalence class in <code>_sortedNodeList</code>.
	 *  <p>Initial score is the product of the equivlance class size and the tree size
	 *  corresponding to the class.
	 *
	 *  @param startIndex       starting index of a new equivalence class
	 *  @param onePastEndIndex  1 + the index of the last member of the equivalence class
	 */
	private void setInitialScores(int startIndex, int onePastEndIndex)
	{
		for (int equivCursor = startIndex; equivCursor < onePastEndIndex; equivCursor++)
		{
			int equivClassSize = onePastEndIndex - startIndex;
			DataNode theNode = (DataNode)_sortedNodeList.get(equivCursor);

			theNode.score = equivClassSize * theNode.treeSize;
			if (theNode.score > _maxInitialScore)
				_maxInitialScore = theNode.score;
		}
	}

	/** Changes initial scores in the list so that the first element of each equivalence class has
	 *  the highest score, and the next element's score is less than the score of any first element
	 *  in equivalance class, and so on.  So the ordering is something like this: <br>
	 *  <code>(equivlance class number, rank in equivalance class)
	 *  (1, 1) (2, 1) (3, 1) (1, 2) (2, 2) (3, 2)</code>
	 */
	private void setRankNodeScores()
	{
		// go through list, and set rank parameters
		Iterator iter = _sortedNodeList.iterator();
		while (iter.hasNext())
		{
			DataNode node = (DataNode) iter.next();
			// nodes that are not the first in rank in their class should go after
			// all nodes that are the first in rank in their class
			node.score += (1 - node.rankInClass) * (_maxInitialScore + 1);

			if (node.score < _minInitialScore)
				_minInitialScore = node.score;
		}

		// go through list, adding a value to each node so that all scores are positive
		iter = _sortedNodeList.iterator();
		while (iter.hasNext())
		{
			DataNode node = (DataNode) iter.next();
			node.score += (-_minInitialScore) + MIN_NODE_SCORE;
		}
	}

	/** Calculates the height of each node, puts nodes in lists based on height.
	 *  <p>Function is called recursively.
	 *  @param root     the root of the subtree
	 *  @return         the height of the subtree
	*/
	private int calculateHeightAndCreateLists(DataNode root)
	{
		DataNode currentNode = root;
		Vector children = _dataGraph.getEdges(currentNode);

		int height = 0;
		int treeSize = 1;

		if (children.size() != 0)
		{
			int maxHeight = 0;
			for (int i = 0; i < children.size(); i++)
			{
				DataNode child = (DataNode)children.get(i);
				int childHeight = calculateHeightAndCreateLists(child);
				if (maxHeight < childHeight)
					maxHeight = childHeight;
				treeSize += child.treeSize;
			}

			height = 1 + maxHeight;
		}

		addToList(currentNode, height);
		currentNode.treeSize = treeSize;
		return height;
	}

	/** Inserts <code>node</code> into the height-i list
	 *  @param  node        the node to insert into a height list
	 *  @param  i           the height of node
	 */
	private void addToList(DataNode node, int i)
	{
		if (i >= _tempNodeList.size()) // need to add a another vector into NodeList
			_tempNodeList.add(new ArrayList());

		ArrayList list = (ArrayList) _tempNodeList.get(i);
		list.add(node);
	}

	/** For debugging.  Prints to <code>out</code> only if <code>FuzzyTreeModel.debugLevel < 5</code>.
	 */
	public void printLists(PrintWriter out)
	{
		if (FuzzyTreeModel.debugLevel < 5)
			return;

		for (int i = 0; i < _tempNodeList.size(); i++)
		{
			ArrayList list = (ArrayList) _tempNodeList.get(i);
			out.println("List " + i + ": ");

			for (Iterator iter = list.iterator(); iter.hasNext(); )
			{
				DataNode node = (DataNode)iter.next();
				out.println(node.isomorphismIndex + "\t" + node.rankInClass + "\t" + node.getLabel().toLowerCase() + ":" +
							   " " + node.getData() + "\tScore: " + node.score);
			}

			out.println();
			out.println();
		}
	}
}